import { ICommand } from '../Interfaces/ICommand';
import { Message, Collection } from 'discord.js';
import { IContext } from '../Interfaces/IContext';
import { ICommandArgument } from '../Interfaces/ICommandArgument';
/**
 * @module Command
 * @preferred
 */
export declare abstract class Command implements ICommand {
    AllowedChannels: string[];
    AllowedRoles: string[];
    AllowedUsers: string[];
    RequiresDatabase: boolean;
    Name: string;
    Namespace: string;
    AllowedGuilds: string[];
    Data: any;
    Blurb: string;
    BotContext: IContext;
    CooldownRate: number;
    LastRun: number;
    Arguments: ICommandArgument[];
    Parametrized: boolean;
    Disabled: boolean;
    Type: string;
    Lockdown: boolean;
    private Modified;
    constructor(AllowedChannels: string[], AllowedRoles: string[], AllowedUsers: string[], RequiresDatabase?: boolean);
    get LocalData(): any;
    set LocalData(value: any);
    get HasLocalData(): boolean;
    abstract Run(message: Message, args?: any): Promise<any>;
    private HasLocalField;
    protected ValidateChannel(channelId: string): boolean;
    protected ValidateRoles(roles: Collection<any, any>): boolean;
    protected ValidateUsers(user: string): boolean;
    private GetValidationPermission;
    ModifyPermissions(type: string, action: string, key: string, local?: boolean): void;
    AddAllowedChannel(channelId: string, local?: boolean): void;
    AddAllowedRole(roleId: string, local?: boolean): void;
    AddAllowedUser(user: string, local?: boolean): void;
    Call(message: Message, isOwner: boolean): Promise<any>;
    /**
     * Returns false if command is on cooldown
     */
    CheckCooldown(): boolean;
    ContextInjection(context?: any): {};
    GetArguments(message: Message): any;
    private GetParameterizedArguments;
    private GetGenericArguments;
    RemoveAllowedChannel(channelId: string, local?: boolean): void;
    RemoveAllowedRole(roleId: string, local?: boolean): void;
    RemoveAllowedUsers(user: string, local?: boolean): void;
    Save(force?: boolean): Promise<any>;
    Validate(message: Message): boolean;
}
//# sourceMappingURL=Command.d.ts.map